# !usr/bin/env python

import sys
sys.path.append('modules')
from ROOT import TFile, TPostScript, TCanvas, TGraphErrors, TLegend, TLatex, TH1, gROOT, gStyle, gPad
from classes import Point, Peak
from functions import FakeGaussian, GetData, FindPeaks, MakeGraph

gROOT.SetStyle("Plain")
gStyle.SetEndErrorSize(0.0); gStyle.SetErrorX(0)
gStyle.SetTitleX(0.1)
gStyle.SetTitleW(0.8)
gStyle.SetTitleBorderSize(0)
gStyle.SetTitleBorderSize(0)
gStyle.SetTitleSize(0.05,"x")
gStyle.SetTitleColor(4)

f1=TFile("AllPythiaJetJet.root")
f1.ls()

dirTemplate = "X_body_decay/"
c1=TCanvas("c1", "BPRE", 0, 0, 800, 500)
psfig="jjmass.eps"
ps1 = TPostScript(psfig,113)

# histoName="inv_JetJet"
h1=f1.Get("jjmass")


#Fake Gaussian
#FakeGaussian(h1, 100, 50, 20000000)
#FakeGaussian(h1, 500, 50, 20000000)
FakeGaussian(h1, 1000, 20, 200000)
FakeGaussian(h1, 1500, 50, 30000)
FakeGaussian(h1, 2800, 40, 1200)


minSig=7

Ymin=1
Ymax=6000000
Xmin=0 
Xmax=4000 

h=gPad.DrawFrame(Xmin,Ymin,Xmax,Ymax);
ay=h.GetYaxis()
ay.SetLabelSize(0.03)
ay.SetTitleSize(0.04)
ay.SetNdivisions(505)
ay.SetTitle( "Events" );

ax=h.GetXaxis()
ax.SetLabelSize(0.03)
ax.SetTitleSize(0.04)
ax.SetTitle( "M(jet-jet) [GeV]" )
ax.SetTitleColor(1)

ax.Draw("same")
ay.Draw("same")
gPad.SetLogy(1)
gPad.SetLogx(0)


bckg = TGraphErrors(h1)
data = GetData(h1)


## 7-sigma approach 
peaks = FindPeaks(data, 1, minSig) #returns a list of Peak objects (each peak contains a list of Point objects)

"""
print 'RESULTS OF PEAK SEARCH:\n'
for i in range(len(peaks)):
    print 'Peak Number:', peaks[i].GetPeakNumber(), 'Peak Start:', peaks[i].GetPeakStart().x, 'Peak End:', peaks[i].GetPeakEnd().x, 'Peak Stat Sig:', peaks[i].GetStatSig(), 'Residuals:', peaks[i].GetResiduals(), '\n'
    print peaks[i].GetPeakStart().GetYErrLow(), peaks[i].GetPeakEnd().GetYErrLow() 
print 'END OF PEAK SEARCH RESULTS'
"""

peak,lreg = MakeGraph(peaks)

bckg.SetMarkerSize(0.5)
bckg.SetMarkerColor(1)
bckg.SetMarkerStyle(20)
bckg.Draw('P')


# always ignore the first point
for i in range(1,len(peak)):


 if (peaks[i].GetStatSig() <minSig): continue 

 print 'Peak Number:', peaks[i].GetPeakNumber(), 'Peak Start:', peaks[i].GetPeakStart().x, 'Peak End:', peaks[i].GetPeakEnd().x, 'Peak Stat Sig:', peaks[i].GetStatSig(), 'Residuals:', peaks[i].GetResiduals(), '\n'
 print peaks[i].GetPeakStart().GetYErrLow(), peaks[i].GetPeakEnd().GetYErrLow()


 peakG=peak[i]
 peakG.SetMarkerColor(4)
 peakG.SetMarkerStyle(23)
 peakG.SetMarkerSize(0.8)
 peakG.Draw('P')

 lreG=lreg[i]
 lreG.SetMarkerColor(2)
 lreG.SetMarkerStyle(23)
 lreG.SetLineColor(2)
 lreG.Draw('L')

 l = TLatex()
 l.SetTextAngle(45)
 l.SetTextSize(0.04)
 x = peaks[i].GetPeakEnd().GetX()
 y = peaks[i].GetPeakEnd().GetY()
 ssig = '#sigma = ' + str(round(peaks[i].GetStatSig(), 1))
 l.DrawLatex(x + 100, y + 100, ssig)


leg2=TLegend(0.55, 0.74, 0.8, 0.87);
leg2.SetBorderSize(0);
leg2.SetTextFont(62);
leg2.SetFillColor(10);
leg2.SetTextSize(0.035);
leg2.AddEntry(peakG,"Peak","p")
leg2.AddEntry(lreG,"Background","l")
leg2.Draw("same");


c1.Update()
ps1.Close()

# pause before exiting (to view graph)
print 'Press any key to exit'
raw_input()



