def LinearRegression(peaks_list):
    """
    Takes a list of Peak objects and linearly regresses the points in each peak to form a background
    """
    
    for i in range(len(peaks_list)):
        
        points = deepcopy(peaks_list[i].GetPoints())
        peak_start = peaks_list[i].GetPeakStart()
        peak_end = peaks_list[i].GetPeakEnd()
        
        x1 = peak_start.GetX()
        y1 = peak_start.GetY()
        y1_err = peak_start.GetYErrHigh()
        
        x2 = peak_end.GetX()
        y2 = peak_end.GetY()
        y2_err = peak_end.GetYErrHigh()
        
        m = ((y2 + y2_err) - (y1 + y1_err))/(x2 - x1)
        b = y1 + y1_err - (m * x1)
        
        for j in range(len(points)):
            
            linreg_point = deepcopy(points[j]) #copy all attributes of the point to the linearly regressed point
            x = points[j].GetX()
            y_new = (m * x) + b #calculate new y value for linearly regressed point
            linreg_point.SetY(y_new) # set new y value
            peaks_list[i].AddLinRegPoint(linreg_point)
            
def Residuals(peaks_list):
    """
    Takes a list of Peak objects and calculates the residuals between the original Point objects and the regressed Point objects in each Peak object
    """ 
    
    for i in range(len(peaks_list)):
        points =  deepcopy(peaks_list[i].GetPoints())  
        linreg_points =  deepcopy(peaks_list[i].GetLinRegPoints())
        
        for j in range(len(points)):
            y = points[j].GetY()
            y_err = points[j].GetYErrLow()
            linreg_y = linreg_points[j].GetY()
            
            residual = y - y_err - linreg_y
            peaks_list[i].AddResidual(residual)
            
def StatisticalSignificance(peaks_list):
    """
    Takes a list of Peak objects and calculates the statistical significance of each Peak object
    """
    
    for i in range(len(peaks_list)):
        residuals = peaks_list[i].GetResiduals()
        N = 0 #reset dA value (sum of residuals)
            
        for j in range(len(residuals)):
            N += residuals[j]
                
        if N <= 0:
            peaks_list[i].SetStatSig(0)   
        else:
            peaks_list[i].SetStatSig( N / sqrt(N) )
